#include <stdio.h>
#include "va108xx.h"
#include "reb_log.h"
#include "reb_board.h"
#include "driver_common.h"
#include "reb_timer.h"
#include "gpio_va108xx.h"




#define TIM_FUNC_01   0x1

/********************************
*
*
*
**********************************/
int CONFIG_TIM0(void)
{
	
	    VOR_GPIO->BANK[0].IRQ_ENB = 1<< 12   ;  // set PA[12] Enable bit 
	
	
			VOR_TIM0->RST_VALUE = 0x99 ; 
	    VOR_TIM0->CNT_VALUE = 0x88   ; 
	    VOR_TIM0->ENABLE = 0x88   ;  
	    VOR_TIM0->CSD_CTRL = 0x88   ;  
			VOR_TIM0->CASCADE0 = 0x99 ; 
			VOR_TIM0->CASCADE1 = 0x99 ; 
			VOR_TIM0->CASCADE2 = 0x99 ; 
	    VOR_TIM0->PWMA_VALUE = 0x88   ;  
	    VOR_TIM0->PWMB_VALUE = 0x88   ; 	
	    VOR_TIM0-> CTRL = 0x88  ;  
	    return(0)  ;  
}	

/********************************
*  Input capture function - Uses timer to measure time a button is pressed
*   a) configure port pin as input and enable interrupt
*   b) configure TIM channel as input 
**********************************/

#define  FUNCSEL1 0x1     //  
#define PWM_period 2500
#define STAT_SEL_1CYC  0 
#define STAT_SEL_ACTIVE 1
#define STAT_SEL_TOGGLE 2
#define STAT_SEL_PWMA 3
#define STAT_SEL_PWMB 4 
#define STAT_SEL_ENAB  5
#define STAT_SEL_PWM_ACTIVE 6 


//*************************************************************************
//  Routine to configure TIM1 as PWMA mode 
//    -  Cascade inputs not used  
//    -  RESET_VALUE set to "PWM_period*2" = 5000
//    -  PWM edge aligned with PWMA = 90% of RESET VALUE 
//**************************************************************************
uint32_t CONFIG_TIM1_PWMA(void)
{
	    VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE = 0xFFFFFFFF  ;  //  enable all peripheral clocks 
			VOR_SYSCONFIG->TIM_CLK_ENABLE |= 0xFFFF  ;  //  enable clocks to lower 16 TIM blocks 
	
			VOR_IOCONFIG->PORTA[1]= IOCONFIG_PORTA_PLEVEL_Msk | IOCONFIG_PORTA_PEN_Msk |(FUNCSEL1 << IOCONFIG_PORTA_FUNSEL_Pos); // set pin func to TIM1
		
			VOR_GPIO->BANK[0].DIR |= (1 << 1);    // set PA[1] to an output 

	    VOR_TIM1->CTRL = (TIM1_CTRL_IRQ_ENB_Msk | ( STAT_SEL_PWMA << TIM1_CTRL_STATUS_SEL_Pos))  ;  //  
			VOR_TIM1->RST_VALUE = (PWM_period*2) - 1 ; 
	    VOR_TIM1->CNT_VALUE = 0 ; 
	
//     VOR_TIM1->CSD_CTRL |= ( TIM1_CSD_CTRL_CSDEN0_Msk ) ;  // 
//	    VOR_TIM1->CSD_CTRL |= ( TIM1_CSD_CTRL_CSDEN0_Msk | TIM1_CSD_CTRL_CSDINV0_Msk ); // no cascade for TIM1
//			VOR_TIM1->CASCADE0 = (TIM_CAS_SRC_PORTA_2) ;   // set cascade 0 source to PORTA2
//			VOR_TIM1->CASCADE1 = 0x00 ; 
//			VOR_TIM1->CASCADE2 = 0x00 ; 
        VOR_TIM1->PWMA_VALUE = .9* (PWM_period*2) ;  
//	    VOR_TIM1->PWMB_VALUE = 0x88   ; 	
//	    VOR_TIM1-> CTRL = 0x88  ;  

			VOR_TIM1->ENABLE = 0x1   ;    // enable TIM1
			return(0) ;    
}
//*************************************************************************
//  Routine to configure TIM2 as PWMB mode 
//    -  Cascade 0 selects PA11 (push button)  
//    -  RESET_VALUE set to "PWM_period" = 2500
//    -  PWM centered with PWMA = 40% and PWMB = 60% of PWM_period     
//**************************************************************************
uint32_t CONFIG_TIM2_PWMB(void)
{
	    VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE = 0xFFFFFFFF  ;  //  enable all peripheral clocks 
			VOR_SYSCONFIG->TIM_CLK_ENABLE |= 0xFFFF  ;  //  enable clocks to lower 16 TIM blocks 
	
			VOR_IOCONFIG->PORTA[2]= IOCONFIG_PORTA_PLEVEL_Msk | IOCONFIG_PORTA_PEN_Msk |(FUNCSEL1 << IOCONFIG_PORTA_FUNSEL_Pos); // set pin func to TIM1
		
			VOR_GPIO->BANK[0].DIR |= (2 << 1);    // set PA[2] to an output 

#define TIM_CAS_SRC_PORTA_11 11   // 
	    VOR_TIM2->CTRL = (TIM1_CTRL_IRQ_ENB_Msk | ( STAT_SEL_PWMB << TIM1_CTRL_STATUS_SEL_Pos))  ;  //  
			VOR_TIM2->RST_VALUE = PWM_period - 1 ; 
	    VOR_TIM2->CNT_VALUE = PWM_period - 1  ; 
	
		  VOR_TIM2->CSD_CTRL |= ( TIM1_CSD_CTRL_CSDEN0_Msk  | TIM1_CSD_CTRL_CSDTRG0_Msk ) ;  
												// Setup cascade control so timer triggers when PA11(button) is pressed
			VOR_TIM2->CASCADE0 = (TIM_CAS_SRC_PORTA_11) ;   // set cascade 0 source to PORTA11
			VOR_TIM2->CASCADE1 = 0x00 ; 
			VOR_TIM2->CASCADE2 = 0x00 ; 
      VOR_TIM2->PWMA_VALUE = PWM_period * .9   ;  
	    VOR_TIM2->PWMB_VALUE = PWM_period * .1	; 	
 
			VOR_TIM2->ENABLE = 0x1   ;    // enable TIM2
			
	return(PWM_period)  ;  
}	
//*************************************************************************
//  Routine to configure TIM3 as PWMB mode 
//    -  Cascade 0 selects PA11 (push button)  
//    -  RESET_VALUE set to "PWM_period" = 2500
//    -  PWM centered with PWMA = 40% and PWMB = 60% of PWM_period   
//    
//**************************************************************************
#define TIM_CAS_SRC_PORTA_11 11   ///
uint32_t CONFIG_TIM3_PWMB(void)
{
	    VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE = 0xFFFFFFFF  ;  //  enable all peripheral clocks 
			VOR_SYSCONFIG->TIM_CLK_ENABLE |= 0xFFFF  ;  //  enable clocks to lower 16 TIM blocks 
	
			VOR_IOCONFIG->PORTA[3]= IOCONFIG_PORTA_PLEVEL_Msk | IOCONFIG_PORTA_PEN_Msk |(FUNCSEL1 << IOCONFIG_PORTA_FUNSEL_Pos); // set pin func to TIM1
		
			VOR_GPIO->BANK[0].DIR |= (3 << 1);    // set PA[3] to an output 
  
	    VOR_TIM3->CTRL = (TIM1_CTRL_IRQ_ENB_Msk | ( STAT_SEL_PWMB << TIM1_CTRL_STATUS_SEL_Pos))  ;  //  
			VOR_TIM3->RST_VALUE = PWM_period - 1 ; 
	    VOR_TIM3->CNT_VALUE = PWM_period - 1; 
	
		  VOR_TIM3->CSD_CTRL |= ( TIM1_CSD_CTRL_CSDEN0_Msk | TIM1_CSD_CTRL_CSDTRG0_Msk ) ;  
												// Setup cascade control so timer triggers when PA11(button) is pressed
			VOR_TIM3->CASCADE0 = (TIM_CAS_SRC_PORTA_11) ;   // set cascade 0 source to PORTA11
			VOR_TIM3->CASCADE1 = 0x00 ; // not used 
			VOR_TIM3->CASCADE2 = 0x00 ; // not used 
      VOR_TIM3->PWMA_VALUE = PWM_period * .6   ;  
	    VOR_TIM3->PWMB_VALUE = PWM_period * .4  ; 	 
			VOR_TIM3->ENABLE = 0x1   ;    // enable TIM3
			
	return(PWM_period)  ;  
}	
//*****************************************************
//  Routine to configure the push button switch as an edge sensitive interrupt
//    -   pullup enable, digital filter set for div32 clock * 5 samples  = 
//    -    interrupt only on falling edge. 

#define user_button_pin  11 

uint32_t setup_pushbutton(uint32_t debounce_count)
{
			VOR_SYSCONFIG->IOCONFIG_CLKDIV1 = 0x20  ;  //  set clkdiv0 to div32
			
			VOR_IOCONFIG->PORTA[11]= (IOCONFIG_PORTA_PLEVEL_Msk | IOCONFIG_PORTA_PEN_Msk); // 
//			VOR_IOCONFIG->PORTA[11]= ((debounce_count<<IOCONFIG_PORTA_FLTTYPE_Pos) | (1<<IOCONFIG_PORTA_FLTCLK_Pos) | IOCONFIG_PORTA_PLEVEL_Msk | IOCONFIG_PORTA_PEN_Msk );
		  	VOR_IOCONFIG->PORTA[11]= ((5<<IOCONFIG_PORTA_FLTTYPE_Pos) | (1<<IOCONFIG_PORTA_FLTCLK_Pos) | IOCONFIG_PORTA_PLEVEL_Msk | IOCONFIG_PORTA_PEN_Msk );
		
      VOR_GPIO->BANK[0].DIR &= !(1 << user_button_pin)  ;  //  set PA11 as input
	    VOR_GPIO->BANK[0].IRQ_EVT &= ~(1 << user_button_pin)   ; //  set to zero for interrupt on H->L transition 
			VOR_GPIO->BANK[0].IRQ_SEN &= ~(1 << user_button_pin)   ; //  set to zero for interrupt on sig transition
			VOR_GPIO->BANK[0].IRQ_EDGE |= 1 << user_button_pin     ;  //  set to 1 
//	    VOR_GPIO->BANK[0].IRQ_ENB |= 1 << user_button_pin      ;  // set PA[11] Enable bit 
	
  return(0)  ;  
}

//*****************************************************
//  ** routine to pull PA9 low  for a time dependent on the input variable  

uint32_t  pulse_PA9_low(uint32_t count)
{

			uint32_t i  ; 
			i = count ;  
			VOR_GPIO->BANK[0].DATAOUT |= (1 << 9);    // set PA[9]  output to 1 
	 		VOR_GPIO->BANK[0].DIR |= (1 << 9);    // set PA[9] to an output 
	
	    VOR_GPIO->BANK[0].DATAOUT &= ~(1 << 9);    // set PA[9]  output to 0 
	    for(i=0;i<count;i++)  { VOR_GPIO->BANK[0].DATAOUT &= ~(1 << 9);  }  
	
	     VOR_GPIO->BANK[0].DATAOUT |= (1 << 9);    // set PA[9]  output to 1 
			return(0)  ;	 
}

/***************************************************************************************
 * @file     reb_main.c
 * @version  V1.0
 * @date     31. March 2016
 *
 * @note
 * VORAGO Technologies 
 *
 * @note
 * Copyright (c) 2013-2016 VORAGO Technologies. 
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY 
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT. 
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES 
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL 
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/


int main()
{  
	uint32_t i ;  
//	VOR_TIM_SIGNAL s0, s8; 
	
	/* Enable clocks for all peripherals */ 
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE = ( CLK_ENABLE_PORTA | CLK_ENABLE_PORTB | CLK_ENABLE_SPIA | CLK_ENABLE_SPIB | CLK_ENABLE_SPIC |
																					 CLK_ENABLE_UARTA | CLK_ENABLE_UARTB | CLK_ENABLE_I2CA | CLK_ENABLE_I2CB | CLK_ENABLE_IRQSEL |
                                           CLK_ENABLE_IOMGR | CLK_ENABLE_UTILITY | CLK_ENABLE_PORTIO | CLK_ENABLE_SYSTEM ); 
	
	
		setup_pushbutton(50)  ;     //   setup PA11 (push button on REB1) to generate an interrupt when pressed 
	
	i = CONFIG_TIM1_PWMA()  ;  //  This will setup TIM1 and it will immediately begin output pulses

	i = CONFIG_TIM2_PWMB()   ;  //  This will setup TIM2 but it will not start until button is pressed 
	i = CONFIG_TIM3_PWMB()   ; //  This will setup TIM1 but it will not start until button is pressed 

	VOR_GPIO->BANK[0].IRQ_ENB |= 1 << user_button_pin    ; 	
	
	
	while(1)    //    for debugging purposes, loop here forever  
	{
		i ++  ;    //   loop here forever
	}
	
	
	
}
